package tst;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

import javax.jms.JMSException;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.naming.Context;
import javax.naming.InitialContext;

public class Test {

	public static void main(String[] args) throws Exception{		
		
		Thread t1 = createThread("[my_ip]", 4447);
		Thread t2 = createThread("[my_ip]", 4449);
		t1.start();
		t2.start();
		t1.join();
		t2.join();
	}	
	
	private static Thread createThread(final String host, final int port){
		Thread t = new Thread(new Runnable() {
			
			public void run() {
				try {
					Context ctx = getContext(host, port);					
					final Queue q = (Queue)ctx.lookup("jms/queue/test");
					final QueueConnectionFactory factory = (QueueConnectionFactory)ctx.lookup("jms/RemoteConnectionFactory");
					produceMessages(factory, q, true, false);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
		return t;
	}
	
	private static Context getContext(String host, int port) throws Exception{
		
        String jndiURL = "remote://" + host + ":" + port;
	    Hashtable<String, String> jndiProps = new Hashtable<String, String>();
	    jndiProps.put("java.naming.provider.url", jndiURL);
	    jndiProps.put(Context.INITIAL_CONTEXT_FACTORY, "org.jboss.naming.remote.client.InitialContextFactory"); 
	    jndiProps.put(Context.SECURITY_PRINCIPAL, "App1");
	    jndiProps.put(Context.SECURITY_CREDENTIALS, "App2");
	    
	    return new InitialContext(jndiProps);      
	}
	
	public static void produceMessages(QueueConnectionFactory factory, Queue q, boolean batched, boolean singleConnection) throws Exception
	{
				
		List<QueueConnection> cons = new ArrayList<QueueConnection>();
		int messageNumber = 1000;
		int numberOfThreads = 10;
		try{
			ExecutorService service = Executors.newCachedThreadPool();
			
			if (singleConnection){
				QueueConnection qcon = factory.createQueueConnection("App1", "App2");
				qcon.start();
				cons.add(qcon);
			}
			for (int i = 0; i < numberOfThreads; i++){
				QueueConnection qcon = factory.createQueueConnection("App1", "App2");
				qcon.start();
				cons.add(qcon);
			}
			long start = System.currentTimeMillis();
			for (int i = 0; i < numberOfThreads; i++){				
				QueueConnection qcon = null;
				if (singleConnection){
					qcon = cons.get(0);
				} else{
					qcon = cons.get(i);
				}				
				service.execute(new ProducingThread(q, qcon, batched, messageNumber));
			}	
			service.shutdown();
			service.awaitTermination(1, TimeUnit.DAYS);
			long end = System.currentTimeMillis();
			System.out.println("duration = " + (end-start));
			System.out.println("messages/second = " + numberOfThreads*messageNumber*1000f/(end-start));						
		} catch (Exception e){
			e.printStackTrace();
		} finally{
			if (cons != null){
				for (QueueConnection con : cons){
					con.close();
				}
			}
		}		
	}
}

class ProducingThread extends Thread{
	
	private Queue queue;
	private QueueConnection queueConnection;
	private int count;
	private boolean batched;
	
	public ProducingThread(Queue queue, QueueConnection queueConnection, boolean batched, int count){
		this.queue = queue;
		this.queueConnection = queueConnection;
		this.batched = batched;
		this.count = count;
	}

	public void run(){
		try {
			produce(queueConnection, queue, batched, 50, count);			 	
			
		} catch (Exception e) {
			e.printStackTrace();
		} 
		System.out.println("thread " + getId() + " has ended");
		return;
	}
	
	public void produce(QueueConnection queueConnection, Queue queue, boolean batched, int batchSize,
			int numberOfMessages) throws Exception{
		
		QueueSession queueSession = null;
		try{
			queueSession = queueConnection.createQueueSession((batched)?true:false, 
					(batched)?Session.SESSION_TRANSACTED:Session.AUTO_ACKNOWLEDGE);
	        produce(queueSession, queue, numberOfMessages, batched, batchSize);
	        /*if (batched){
	        	queueSession.commit();
	        }*/
		}
		finally{
			if (queueSession != null){
        		try {
					queueSession.close();
				} catch (JMSException e) {
                    System.out.println("Closing error: " + e.toString());
				}
        	}
		}
	}
	
	public void produce(QueueSession queueSession, Queue queue, int numberOfMessages, boolean batched, int batchSize) throws Exception{
		QueueSender queueSender = queueSession.createSender(queue);
        produce(queueSession, queueSender, queue, numberOfMessages, batched, batchSize);
        queueSender.close();
	}
	
	public void produce(QueueSession queueSession, QueueSender queueSender, Queue queue, int numberOfMessages,
			boolean batched, int batchSize) throws Exception{
		TextMessage message = queueSession.createTextMessage();
        for (int i = 0; i < numberOfMessages; i++) {
        	message.setJMSReplyTo(queue);
        	message.setLongProperty("prop_long", 451l);
        	message.setStringProperty("prop_str", "str1!");
            message.setText("Mes " + (i + 1));
            queueSender.send(message);
            if (i !=0 && batchSize !=0 && (i+1)%batchSize==0 && batched){
            	queueSession.commit();
            }
        }
	}
}



